ALTER TABLE tab1 ADD CONSTRAINT foo FOREIGN KEY (id1) REFERENCES tab2 (id2) DEFERRABLE INITIALLY DEFERRED
=
ALTER TABLE tab1 ADD CONSTRAINT foo FOREIGN KEY (id1) REFERENCES tab2 (id2) DEFERRABLE INITIALLY DEFERRED
