move all foo
=
MOVE FORWARD ALL IN foo

move next in foo
=
MOVE NEXT IN foo

move forward 1 from foo
=
MOVE NEXT IN foo

move 1 from foo
=
MOVE NEXT IN foo

move backward 1 from foo
=
MOVE PRIOR IN foo

move prior in foo
=
MOVE PRIOR IN foo

move absolute 1 foo
=
MOVE FIRST IN foo

move absolute -1 foo
=
MOVE LAST IN foo

move absolute 42 foo
=
MOVE ABSOLUTE 42 IN foo
